/*
 ** Created on 2020/05/07.
 */
#include <stdbool.h>
#include <elf_sign.h>
#ifndef _DEB_SIGN_H
#define _DEB_SIGN_H
#ifdef __cplusplus
extern "C" {
#endif
typedef struct sof_ctx_t sof_ctx_t;

// #define TEMP_CERT            "/tmp/_keylin.crt"
extern char TEMP_CERT[256];

#define SIGN_FILE            "_kylin.sig"
#define CERT_FILE            "_kylin.crt"
enum SignModel
{
    ca_moclose=1, ca_warning, ca_prevent
};
enum interface_auth_result {
        IF_AUTH_UNKNOWN = -1,   /* 未知认证结果，表示还没进行认证 */
        IF_AUTH_SUCCESS = 0,    /* 认证成功 */
        IF_AUTH_FAILED,         /* 认证失败 */
        IF_AUTH_ERROR           /* 认证出错 */
};

#define interface_control_file "/etc/kylinsign/interface_control.conf"
enum NESTING
{
    not_need_nest = 0,
    is_need_nest
};
/*
 *函数名称：kysec_interface_authentication
 *输入参数：inner，hash
 *当@inner值为1时，表示内部接口认证，此时@hash需要填充正确的hash值
 *当@inner值不为1时，表示外部接口认证，此时@hash可以为NULL
 *hash值为获取到的当前时间取摘要后与kysec_flag进行&运算得到的结果
 *输出参数：AUresult
 *此参数为elf文件验证后获得的验证结果与证书相关信息，具体结构体定义见elf_sign.h
 *在函数内部会自动为结构体中的serialnum与finger分配内存，此内存需函数调用者自行在函数外部释放
 *返回值：返回验证结果
 *返回值为0，表示验证成功，返回值为1，表示验证失败,返回值为2，表示验证出错
*/
int kysec_interface_authentication(int inner, unsigned char *hash,AU_get *AUresult);
int get_cert_sec_type(char *outd);
int getCertSecType(char *CertSecType,const char *user_cert);
int control_dpkg_file(int type);
void output_error_msg(int ret);
int change_interface_control_conf(char *newstring);
int getDebCert(const char *deb);
int operatesign(char *buf,int len,int num);
void getnext(char t[], int next[]);
int certKMP(char *certfile);
int getDebCertNum(const char *deb,char *oname,int num);
int getDebCertSerNum(const char *deb,char *user_cert);
int doDebVerify2(const char *deb,char *oname,bool isinstall);
int doDebVerifyall(const char *deb,bool isinstall);
int doDebSign(const char *deb, const char *certPath, const char *pkey_file);
int Multiple_doDebSign(const char *deb, const char *certPath, const char *pkey_file);
int doDebVerify(const char *deb);
int getAllOu(const char *deb,char *ouchain);
int getFirstDebCert(const char *deb);
int getFirstCertSecType(const char *deb,char *CertSecType);
int getIssuerST(const char *deb,char *IssuerST);
int getSubjectO(const char *deb,char *SubjectO);
int getSubjectOU(const char *deb,char *SubjectOU);
int getIssuerOU(const char *deb,char *IssuerOU);
int getSubjectCN(const char *dev,char *SubjectCN);
int SignPackageAll(const char *debPath, const char *certPath, const char *pkey_file, int need_nest);
int  SignPackageAllElf(const char *debPath, const char *certPath, const char *pkey_file);
int findElfSign(const char *path, int recursive, const char *certPath, const char *pkey_file);
int findDebSign(const char *path, int recursive, const char *certPath, const char *pkey_file);
int check_file_sign(char *fpath);
int deleteElfSign(const char *path, int recursive);
extern int GetConfigValue(const char * confPath, char * optionName, char *optionData, unsigned int optionDataLen);
int verifybfsign(const char * deb,const char *certPath,const char * signPath);
int verify_cert_valid(sof_ctx_t *ctx);
int DelPackageAll(const char *debPath);
int dodebsign(sof_ctx_t *ctx, const char *deb);
int multiple_dodebsign(sof_ctx_t *ctx, const char *deb);
int signpackageall(sof_ctx_t *ctx, const char *deb, int need_nest);
int  signpackageallelf(sof_ctx_t *ctx, const char *deb);
int dodebverifyall(sof_ctx_t *ctx, const char *deb, bool isinstall);
void usb_change_callback(void* ctx, int action);
int dodebverify2(sof_ctx_t *ctx, const char *deb,char *oname,bool isinstall);
int findelfsign(const char *path, int recursive, const char *certPath, sof_ctx_t *ctx);
int finddebsign(const char *path, int recursive, const char *certPath, sof_ctx_t *ctx);
int BJCA_dodebverify(sof_ctx_t *ctx, const char *deb, bool isinstall);
int getDebCertAllInfo(const char *deb,char *info,char *CertObject);
int getAllIssuerST(const char *deb,char *IssuerST);
int getAllIssuerOU(const char *deb,char *IssuerOU);
int getAllSubjectOU(const char *deb,char *SubjectOU);
int getAllSubjectO(const char *deb,char *SubjectO);
int getAllSubjectCN(const char *deb,char *SubjectCN);
int getAllCertSecType(const char *deb,char *CertSecType);
int getCertNumber(const char *deb);
int BJCA_base_dodebverify(sof_ctx_t *ctx, const char *deb, bool isinstall);
int BJCA_one_dodebverify(sof_ctx_t *ctx, const char *deb, bool isinstall);
int BJCA_more_dodebverify(sof_ctx_t *ctx, const char *deb, bool isinstall);
int BJCA_apkverify(const char *filename);
int Compute_file_SM3(const char *file_path, char *sm3_str);
int fileSign(sof_ctx_t *ctx, char *filename, char *signfile);
int fileVerify(sof_ctx_t *ctx, char *filename, char *signfile, char *cert);
/*
 * 参数： 无
 * 返回值：1 关闭；2 警告；3 阻止； -1获取失败；
 */
int get_dpkg_file_info();

/*
 * 参数： 1 关闭；2 警告；3 阻止；其它 错误参数；
 * 返回值：0成功；-3 参数错误； -4 未启用验签插件； 其它 失败；
 */
int control_dpkg_file(int type);

/*
 * 给开明软件签名
 * 参数： 待签名开明软件包
 *返回值：0 签名成功; 1 权限不允许；2 获取开明包签名文件路径失败；3 解压开明包失败；4、签名开明包失败；5 导出证书失败；
 *        6 重新压缩包失败；7 创建签名目录失败； 8 已经存在签名
 */
int do_kaiming_sign(char *ok);

/*
 * 给开明软件验签
 * 参数： 待验签开明软件包
 *返回值：0 验签成功; 1 权限不允许；2 获取开明包签名文件路径失败；3 解压开明包失败；4、验签开明包失败；5 签名文件不存在
 */
int do_kaiming_verify(char *ok);

/*
 * 判断系统是否安装某个软件包
 * 参数： 软件包名
 * 0 已安装，-1出错，1未安装
 */
int is_package_installed(const char *package_name);

//int getIssuerST(const char *deb,char *IssuerST,const char *user_cert,int bio_type);
#ifdef __cplusplus
}
#endif
#endif //_DEB_SIGN_H
