/*************************************************
  FileName: xtxapp.h
  Authro: zhaoyongsheng
  Date: 2017.02.09
  Description:　xtxapp for linux C API declare
*************************************************/

#ifndef _XTXAPP_C_UTIL_H_
#define _XTXAPP_C_UTIL_H_

#ifdef __cplusplus
extern "C" {
#endif

typedef struct sof_ctx_t sof_ctx_t;

#ifndef BASE_STRUCT
	#define BASE_STRUCT
	typedef unsigned int  uint32;
	typedef unsigned char uchar;
	typedef const uchar   kuchar;
	typedef const char    kchar;
#endif

#define SOF_API

/*
 * usbkey change event 
 */
enum {
	e_event_insert = 1,
	e_event_remove = 2,
};

/* 
 * usbkey change callback function prototype
 * @param ctx    : the context for callback, pass by SOF_InitializeEx
 * @param action : change event
 */
typedef void (ukey_change_cb_fn)(void* ctx, int action);

/***************************************************
 * 函数名称 SOF_Initialize
 * 功能说明:初始化应用环境,调用其他接口前要先调用本接口或SOF_InitializeEx
 * 参数说明: 无
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_Initialize(sof_ctx_t **sctx);

/***************************************************
 * 函数名称 SOF_InitializeEx,调用其他接口前要先调用本接口或SOF_Initialize
 * 功能说明:初始化应用环境扩展 设备插拔时可以调用到回调函数 callback_f
 * 参数说明: callback_f [IN] 回调函数的地址指针
 *           ctx        [IN] 回调函数的第一个参数
 * 返回值: 0成功 非0失败 
 ***************************************************/ 
int SOF_API SOF_InitializeEx(sof_ctx_t **sctx, ukey_change_cb_fn *cb, void* ctx);

/***************************************************
 * 函数名称 SOF_Finalize
 * 功能说明: 清空应用环境 调用后其他接口不能调用
 * 参数说明: 无
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_Finalize(sof_ctx_t *sctx);

/***************************************************
 * 函数名称 SOF_GetVersion
 * 功能说明: 获取动态库的版本号 
 * 参数说明: version     [OUT] 返回版本号的缓冲区，返回不超过32字节
 *           version_len [IN]  表示接收输出缓冲区的长度，防止内存越界
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_GetVersion(sof_ctx_t *sctx, char *version, uint32 ver_len);

/*
 * sign method
 */
enum {
	e_alg_sm3_rsa    = 0x00010001,
	e_alg_sha1_rsa   = 0x00010002,
	e_alg_sha256_rsa = 0x00010004,
	e_alg_sm3_sm2    = 0x00020101,
};

/***************************************************
 * 函数名称 SOF_SetSignMethod
 * 功能说明: 设置签名算法
 * 参数说明: sign_algo [IN] 签名算法
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_SetSignMethod(sof_ctx_t *sctx, uint32 sign_algo);

/***************************************************
 * 函数名称 SOF_GetSignMethod
 * 功能说明: 获取签名算法
 * 参数说明: sign_algo [OUT] 签名算法
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_GetSignMethod(sof_ctx_t *sctx, uint32 *sign_algo);

/*
 * encrypt method
 */
enum {
	e_alg_sm1_ecb = 0x00000101,
	e_alg_sm1_cbc = 0x00000102,
	e_alg_sm1_cfb = 0x00000104,
	e_alg_sm1_ofb = 0x00000108,

	e_alg_ssf33_ecb = 0x00000201,
	e_alg_ssf33_cbc = 0x00000202,
	e_alg_ssf33_cfb = 0x00000204,
	e_alg_ssf33_ofb = 0x00000208,

	e_alg_sm4_ecb = 0x00000401,
	e_alg_sm4_cbc = 0x00000402,
	e_alg_sm4_cfb = 0x00000404,
	e_alg_sm4_ofb = 0x00000408,

	// des key length 8
	e_alg_des_ecb = 0x00000801,
	e_alg_des_cbc = 0x00000802,
	e_alg_des_cfb = 0x00000804,
	e_alg_des_ofb = 0x00000808,

	// des key length 16
	e_alg_des_ede_ecb = 0x00001001,
	e_alg_des_ede_cbc = 0x00001002,
	e_alg_des_ede_cfb = 0x00001004,
	e_alg_des_ede_ofb = 0x00001008,

	// des key length 24
	e_alg_des_ede3_ecb = 0x00002001,
	e_alg_des_ede3_cbc = 0x00002002,
	e_alg_des_ede3_cfb = 0x00002004,
	e_alg_des_ede3_ofb = 0x00002008,

	// aes key length 16
	e_alg_aes_128_ecb = 0x00004001,
	e_alg_aes_128_cbc = 0x00004002,
	e_alg_aes_128_cfb = 0x00004004,
	e_alg_aes_128_ofb = 0x00004008,

	// aes key length 24
	e_alg_aes_192_ecb = 0x00008001,
	e_alg_aes_192_cbc = 0x00008002,
	e_alg_aes_192_cfb = 0x00008004,
	e_alg_aes_192_ofb = 0x00008008,

	// aes key length 32
	e_alg_aes_256_ecb = 0x00010001,
	e_alg_aes_256_cbc = 0x00010002,
	e_alg_aes_256_cfb = 0x00010004,
	e_alg_aes_256_ofb = 0x00010008,
};

enum {
	// 加解密类型
	e_encrypt = 0,
	e_decrypt = 1,

	// 填充方式
	e_padding_none  = 0,
	e_padding_pkcs5 = 1,

	//对称算法
	e_sm1_ecb = 0x00000101,
	e_sm1_cbc = 0x00000102,
	e_sm1_cfb = 0x00000104,
	e_sm1_ofb = 0x00000108,
	e_sm1_mac = 0x00000110,

	e_ssf33_ecb = 0x00000201,
	e_ssf33_cbc = 0x00000202,
	e_ssf33_cfb = 0x00000204,
	e_ssf33_ofb = 0x00000210,

	e_sm4_ecb = 0x00000401,
	e_sm4_cbc = 0x00000402,
	e_sm4_cfb = 0x00000404,
	e_sm4_ofb = 0x00000408,
	e_sm4_mac = 0x00000410,

	//自定义扩展软 对称算法
	e_des_ecb = 0x00000801,
	e_des_cbc = 0x00000802,
	e_des_cfb = 0x00000804,
	e_des_ofb = 0x00000808,
	e_des_mac = 0x00000810,

	e_3des_2key_ecb = 0x00001001,
	e_3des_2key_cbc = 0x00001002,
	e_3des_2key_cfb = 0x00001004,
	e_3des_2key_ofb = 0x00001008,
	e_3des_2key_mac = 0x00001010,

	e_3des_3key_ecb = 0x00002001,
	e_3des_3key_cbc = 0x00002002,
	e_3des_3key_cfb = 0x00002004,
	e_3des_3key_ofb = 0x00002008,
	e_3des_3key_mac = 0x00002010,

	e_aes_128_ecb = 0x00004001,
	e_aes_128_cbc = 0x00004002,
	e_aes_128_cfb = 0x00004004,
	e_aes_128_ofb = 0x00004008,
	e_aes_128_mac = 0x00004010,

	e_aes_192_ecb = 0x00008001,
	e_aes_192_cbc = 0x00008002,
	e_aes_192_cfb = 0x00008004,
	e_aes_192_ofb = 0x00008008,
	e_aes_192_mac = 0x00008010,

	e_aes_256_ecb = 0x00010001,
	e_aes_256_cbc = 0x00010002,
	e_aes_256_cfb = 0x00010004,
	e_aes_256_ofb = 0x00010008,
	e_aes_256_mac = 0x00010010
};

/***************************************************
 * 函数名称 SOF_SetEncryptMethod
 * 功能说明: 设置加密算法
 * 参数说明: enc_alog [IN] 加密算法
 * 返回值: 0成功 非0失败 
 ***************************************************/ 
int SOF_API SOF_SetEncryptMethod(sof_ctx_t *sctx, uint32 enc_algo);

/***************************************************
 * 函数名称 SOF_GetEncryptMethod
 * 功能说明: 获取加密算法
 * 参数说明: enc_alog [OUT] 加密算法
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_GetEncryptMethod(sof_ctx_t *sctx, uint32 *enc_algo);

/***************************************************
 * 函数名称 SOF_GetUserList
 * 功能说明: 获取用户列表
 * 参数说明: user_list     [OUT]    返回格式：用户名1||CertID1&&&用户名2||CertID2&&&用户名3||CertID3&&&,以'\0'结尾
 *           user_list_len [IN/OUT] 输入时表示缓冲区的长度 输出时表示user_list的长度 不包括'\0'
 * 返回值: 0成功 非0失败 
 * 备注 
 ***************************************************/
int SOF_API SOF_GetUserList(sof_ctx_t *sctx, char *user_list, uint32 *ul_len);

/***************************************************
 * 函数名称 SOF_ExportUserCert
 * 功能说明: 导出用户证书 默认导出签名证书 无签名证书导出加密证书
 * 参数说明: certid        [IN]     证书ID,以'\0'结尾
 *           user_cert     [OUT]    返回的der编码的证书
 *           user_cert_len [IN/OUT] 输入(user_cert不为NULL)时标识缓冲区的长度 输出时表示user_cert的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_ExportUserCert(sof_ctx_t *sctx, kchar *certid, 
                               uchar *user_cert, uint32 *uc_len);

/***************************************************
 * 函数名称 SOF_ExportExChangeUserCert
 * 功能说明: 导出用户证书 默认导出加密
 * 参数说明: certid        [IN]     证书ID,以'\0'结尾
 *           user_cert     [OUT]    返回的der编码的证书
 *           user_cert_len [IN/OUT] 输入(user_cert不为NULL)时标识缓冲区的长度 输出时表示user_cert的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_ExportExChangeUserCert(sof_ctx_t *sctx, kchar *certid, 
                                       uchar *user_cert, uint32 *uc_len);

/***************************************************
 * 函数名称 SOF_Login
 * 功能说明: 登录 即验证证书密码
 * 参数说明: certid    [IN] 证书ID,以'\0'结尾
 *           user_pass [IN] 证书密码,以'\0'结尾
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_Login(sof_ctx_t *sctx, kchar *certid, kchar *user_pin);

/***************************************************
 * 函数名称 SOF_IsLogin
 * 功能说明: 判断登录 即验证用户是否登录
 * 参数说明: certid    [IN] 证书ID,以'\0'结尾       
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_IsLogin(sof_ctx_t *sctx,uchar * sDeviceSN);

/***************************************************
 * 函数名称 SOF_Logout
 * 功能说明: 用户登出
 * 参数说明: certid [IN] 证书ID,以'\0'结尾
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_Logout(sof_ctx_t *sctx, kchar *certid);

/***************************************************
 * 函数名称 SOF_GetPinRetryCount
 * 功能说明: 获取证书密码的剩余重试次数
 * 参数说明: certid     [IN]  证书ID,以'\0'结尾
 *           retrycount [OUT] 返回证书密码的剩余重试次数
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_GetPinRetryCount(sof_ctx_t *sctx, kchar *certid, uint32 *rc);

/***************************************************
 * 函数名称 SOF_ChangePassWd
 * 功能说明: 修改证书密码
 * 参数说明: certid    [IN] 证书ID,以'\0'结尾
 *           old_pass  [IN] 证书旧密码,以'\0'结尾
 *           new_pass  [IN] 证书新密码,以'\0'结尾
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_ChangePassWd(sof_ctx_t *sctx, kchar *certid, kchar *old_pin, kchar *new_pin);

/*
 * get certificate type
 */
enum {
	e_cert_version = 1,
	e_cert_serialnumber = 2,
	e_cert_sign_method = 3,
	e_cert_issuer_c = 4,
	e_cert_issuer_o = 5,
	e_cert_issuer_ou = 6,
	e_cert_issuer_st = 7,
	e_cert_issuer_cn = 8,
	e_cert_issuer_l = 9,
	e_cert_issuer_e = 10,
	e_cert_valid_not_before = 11,
	e_cert_valid_not_after = 12,
	e_cert_subject_c = 13,
	e_cert_subject_o = 14,
	e_cert_subject_ou = 15,
	e_cert_subject_st = 16,
	e_cert_subject_cn = 17,
	e_cert_subject_l = 18,
	e_cert_subject_e = 19,
	e_cert_publickey = 20,
	e_cert_issuer_uid = 21,
	e_cert_subject_uid = 22,
	e_cert_key_usage = 23,
	e_cert_subject_dn = 33,
	e_cert_issuer_dn = 34,

	// 密钥用法标识
	e_keyusage_none = 0,
	e_keyusage_sign = (1 << 0),
	e_keyusage_enc  = (1 << 1)
};

/***************************************************
 * 函数名称 SOF_GetCertInfo
 * 功能说明: 获取证书信息
 * 参数说明: user_cert     [IN]     Der编码的证书
 *           user_cert_len [IN]     Der编码的证书长度
 *           type          [IN]     信息类型
 *           outd          [OUT]    输出的信息,传NULL时通过outd_len获取长度
 *           outd_len      [IN/OUT] 输入时(outd不为NULL)表示接收信息的缓冲区长度
 *                                  输出时表示信息的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_GetCertInfo(sof_ctx_t *sctx, kuchar *user_cert, uint32 uc_len, 
					        uint32 type, uchar *outd, uint32 *o_len);

/***************************************************
 * 函数名称 SOF_GetCertInfoByOid
 * 功能说明: 通过OID获取证书扩展信息
 * 参数说明: user_cert     [IN]     Der编码的证书
 *           user_cert_len [IN]     Der编码的证书长度
 *           oid           [IN]     OID串,以'\0'结尾
 *           outd          [OUT]    输出的信息,传NULL时通过outd_len获取长度
 *           outd_len      [IN/OUT] 输入时(outd不为NULL)表示接收信息的缓冲区长度
 *                                  输出时表示信息的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_GetCertInfoByOid(sof_ctx_t *sctx, kuchar *user_cert, uint32 uc_len, 
						         kchar *oid, uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_GetCertEntity
 * 功能说明: 获取证书唯一实体标识
 * 参数说明: user_cert     [IN] Der编码的证书
 *           user_cert_len [IN] Der编码的证书长度
 *           cert_entity     [OUT]    输出的证书唯一实体表示,传NULL时通cert_entity_len获取长度
 *           cert_entity_len [IN/OUT] 输入时(cert_entity不为NULL)表示接收签名值的缓冲区长度
 *                                    输出时表示签名值的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_GetCertEntity(sof_ctx_t *sctx, kuchar *user_cert, uint32 uc_len, 
							  uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_ValidateCert
 * 功能说明: 验证证书有效性
 * 参数说明: 
 *           user_cert     [IN] der编码的证书
 *           user_cert_len [IN] 证书长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_ValidateCert(sof_ctx_t *sctx, kuchar *user_cert, uint32 uc_len,uint32 *out_ret);

/*
 * hash alg type
 */
enum {
	// 摘要算法
	e_alg_sm3    = 0x00000001,
	e_alg_sha1   = 0x00000002,
	e_alg_sha256 = 0x00000004,
	e_alg_md5    = 0x00000008,
	e_alg_sha384 = 0x00000010,
	e_alg_sha512 = 0x00000020,
	e_alg_md5_sha1 = 0x00000040
};

/***************************************************
 * 函数名称 SOF_SignData
 * 功能说明: 对输入的数据进行签名
 * 参数说明: certid        [IN]     证书ID,以'\0'结尾
 *           ind           [IN]     待签名的原文数据
 *           ind_len       [IN]     待签名的原文数据长度
 *           signature     [OUT]    输出的签名值,传NULL时通过signature_len获取长度
 *           signature_len [IN/OUT] 输入时(signature不为NULL)表示接收签名值的缓冲区长度
 *                                  输出时表示签名值的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_SignData(sof_ctx_t *sctx, kchar *certid, 
                         kuchar *ind, uint32 ind_len, 
		         uchar *sig, uint32 *o_len);

/***************************************************
 * 函数名称 SOF_VerifySignedData
 * 功能说明: 对输入的数据进行验签
 * 参数说明: user_cert     [IN] Der编码的证书
 *           user_cert_len [IN] Der编码的证书长度
 *           ind           [IN] 待验签的原文数据
 *           ind_len       [IN] 待验签的原文数据长度
 *           signature     [IN] 签名值
 *           signature_len [IN] 签名值的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_VerifySignedData(sof_ctx_t *sctx, uchar *user_cert, uint32 uc_len, 
								 kuchar *ind, uint32 ind_len, 
								 kuchar *sig, uint32 sig_len);

/***************************************************
 * 函数名称 SOF_SignHashData
 * 功能说明: 对输入的数据进行签名
 * 参数说明: certid        [IN]     证书ID,以'\0'结尾
 *           ind           [IN]     待签名的Hash数据
 *           ind_len       [IN]     待签名的Hash数据长度
 *           hash_alg      [IN]     Hash算法
 *           signature     [OUT]    输出的签名值,传NULL时通过signature_len获取长度
 *           signature_len [IN/OUT] 输入时(signature不为NULL)表示接收签名值的缓冲区长度
 *                                  输出时表示签名值的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_SignHashData(sof_ctx_t *sctx, kchar *certid, 
				 kuchar *ind, uint32 ind_len, 
				 uint32 hash_alg,
				 uchar *sig, uint32 *o_len);

/***************************************************
 * 函数名称 SOF_VerifySignedHashData
 * 功能说明: 对输入的数据进行验签
 * 参数说明: user_cert     [IN] Der编码的证书
 *           user_cert_len [IN] Der编码的证书长度
 *           ind           [IN] 待验签的原文数据
 *           ind_len       [IN] 待验签的原文数据长度
 *           hash_alg      [IN] Hash算法
 *           signature     [IN] 签名值
 *           signature_len [IN] 签名值的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_VerifySignedHashData(sof_ctx_t *sctx, uchar *user_cert, uint32 uc_len, 
					 kuchar *ind, uint32 ind_len, uint32 hash_alg, 
					 kuchar *sig, uint32 sig_len);

/***************************************************
 * 函数名称 SOF_SignMessage
 * 功能说明: 对输入的数据进行PKCS7签名
 * 参数说明: certid        [IN]     证书ID,以'\0'结尾
 *           dwFlag        [IN]     传0时为Attached 传1时为Detached
 *           ind           [IN]     待签名的原文数据
 *           ind_len       [IN]     待签名的原文数据长度
 *           signature     [OUT]    输出的PKCS7签名值,传NULL时通过signature_len获取长度
 *           signature_len [IN/OUT] 输入时(signature不为NULL)表示接收签名值的缓冲区长度
 *                                  输出时表示签名值的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_SignMessage(sof_ctx_t *sctx, int dwFlag, kchar *CertID, kuchar *ind, 
				uint32 ind_len,uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_VerifySignedMessage
 * 功能说明: 对输入的数据进行验签
 * 参数说明: signature     [IN] PKCS7签名值
 *           signature_len [IN] PKCS7签名值的长度
 *           ind           [IN] 待验签的原文数据(在签名为Detached时有效)
 *           ind_len       [IN] 待验签的原文数据长度(在签名为Detached时有效)
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_VerifySignedMessage(sof_ctx_t *sctx, kuchar *p7_data, uint32 p7_len,
					kuchar *ind, uint32 ind_len);

/***************************************************
 * 函数名称 SOF_EncryptData
 * 功能说明: 对数据做数字信封加密
 * 参数说明: user_cert     [IN]     Der编码的证书
 *           user_cert_len [IN]     Der编码的证书长度
 *           ind           [IN]     待加密的明文数据
 *           ind_len       [IN]     待加密的明文数据长度
 *           outd          [OUT]     接收密文数据的缓冲区,传NULL时通过outd_len输出参数
 *           outd_len      [IN/OUT] 输入时(outd不为NULL)表示接收密文的缓冲区长度
 *                                  输出时表示密文的长度
 * 返回值: 0成功 非0失败 
 * 备注:本接口加密的数据 SecX组件不能解密
 ***************************************************/
int SOF_API SOF_EncryptData(sof_ctx_t *sctx, uchar *user_cert, uint32 user_cert_len, 
				kuchar *ind, uint32 ind_len, 
				uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_DecryptData
 * 功能说明: 对数据做数字信封解密
 * 参数说明: certid        [IN]     证书ID,以'\0'结尾
 *           ind           [IN]     待解密的密文数据
 *           ind_len       [IN]     待解密的密文数据长度
 *           outd          [OUT]     接收解密后名文数据的缓冲区,传NULL时通过outd_len输出参数
 *           outd_len      [IN/OUT] 输入时(outd不为NULL)表示接收明文的缓冲区长度
 *                                  输出时表示明文的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_DecryptData(sof_ctx_t *sctx, kchar *certid, kuchar *ind, 
				uint32 ind_len, uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_PubKeyEncrypt
 * 功能说明: 用证书中的公钥进行加密
 * 参数说明: user_cert     [IN]     Der编码的证书
 *           user_cert_len [IN]     Der编码的证书长度
 *           ind           [IN]     待加密的明文数据
 *           ind_len       [IN]     待加密的明文数据长度
 *           outd          [OUT]    接收密文数据的缓冲区,传NULL时通过outd_len输出参数
 *           outd_len      [IN/OUT] 输入时(outd不为NULL)表示接收密文的缓冲区长度
 *                                  输出时表示密文的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_PubKeyEncrypt(sof_ctx_t *sctx, kuchar *user_cert, uint32 user_cert_len, 
				  kuchar *ind, uint32 ind_len, 
				  uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_PriKeyDecrypt
 * 功能说明: 对数据做数字信封解密
 * 参数说明: certid        [IN]     证书ID,以'\0'结尾
 *           ind           [IN]     待解密的密文数据
 *           ind_len       [IN]     待解密的密文数据长度
 *           outd          [OUT]    接收解密后名文数据的缓冲区,传NULL时通过outd_len输出参数
 *           outd_len      [IN/OUT] 输入时(outd不为NULL)表示接收明文的缓冲区长度
 *                                  输出时表示明文的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_PriKeyDecrypt(sof_ctx_t *sctx, kchar *certid, 
				  kuchar *ind, uint32 ind_len, 
				  uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_SymEncryptData
 * 功能说明: 对称加密数据
 * 参数说明: key      [IN]     对称密钥
 *           key_len  [IN]     对称密钥的长度
 *           ind      [IN]     待加密的明文数据
 *           ind_len  [IN]     待加密的明文数据长度
 *           outd     [OUT]    接收密文数据的缓冲区,传NULL时通过outd_len输出参数
 *           outd_len [IN/OUT] 输入时(outd不为NULL)表示接收密文的缓冲区长度
 *                             输出时表示密文的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_SymEncryptData(sof_ctx_t *sctx, kuchar *key, uint32 key_len, 
				   kuchar *ind, uint32 ind_len, 
				   uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_SymDecryptData
 * 功能说明: 对称解密数据
 * 参数说明: key      [IN]     对称密钥
 *           key_len  [IN]     对称密钥的长度
 *           ind      [IN]     待解密的密文数据
 *           ind_len  [IN]     待解密的密文数据长度
 *           outd     [OUT]    接收解密后名文数据的缓冲区,传NULL时通过outd_len输出参数
 *           outd_len [IN/OUT] 输入时(outd不为NULL)表示接收明文的缓冲区长度
 *                             输出时(outd为NULL)表示明文的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_SymDecryptData(sof_ctx_t *sctx, kuchar *key, uint32 key_len, 
				   kuchar *ind, uint32 ind_len, 
				   uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_GenRandom
 * 功能说明: 产生随机数
 * 参数说明: rnd_len [IN]  要产生的随机数长度
 *           rnd     [OUT] 产生的随机数
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_GenRandom(sof_ctx_t *sctx, uint32 rnd_len, kuchar *rnd);

/***************************************************
 * 函数名称 SOF_HashData
 * 功能说明: 数据摘要
 * 参数说明: hash_algo [IN]     摘要算法
 *           ind       [IN]     明文数据
 *           ind_len   [IN]     明文数据的长度
 *           cert      [IN]     参与摘要运算(签名者)的证书或公钥
 *           cert_len  [IN]     参与摘要运算(签名者)的证书或公钥长度
 *           id        [IN]     参与摘要运算(签名者)的ID
 *           id_len    [IN]     参与摘要运算(签名者)的ID长度
 *           outd      [OUT]    摘要后的数据,传NULL时，通过outd_len返回长度
 *           outd_len  [IN/OUT] 输入时(outd不为NULL)表示摘要数据的缓冲区长度
 *                              输出时表示摘要数据长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_HashData(sof_ctx_t *sctx, uint32 hash_algo, kuchar *ind, uint32 ind_len, 
				 kuchar *cert, uint32 cert_len, kuchar *id, uint32 id_len, 
				 uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_HashFile
 * 功能说明: 文件摘要
 * 参数说明: hash_algo [IN]     摘要算法
 *           inf       [IN]     待摘要的文件全路径,已'\0'结尾
 *           cert      [IN]     参与摘要运算(签名者)的证书或公钥
 *           cert_len  [IN]     参与摘要运算(签名者)的证书或公钥长度
 *           id        [IN]     参与摘要运算(签名者)的ID
 *           id_len    [IN]     参与摘要运算(签名者)的ID长度
 *           outd      [OUT]    摘要后的数据,传NULL时，通过outd_len返回长度
 *           outd_len  [IN/OUT] 输入时(outd不为NULL)表示摘要数据的缓冲区长度
 *                              输出时表示摘要数据长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_HashFile(sof_ctx_t *sctx, uint32 hash_algo, kchar *inf, 
				 kuchar *cert, uint32 cert_len, kuchar *id, uint32 id_len, 
				 uchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_Base64Encode
 * 功能说明: 数据Base64编码
 * 参数说明: ind      [IN]     原文数据
 *           ind_len  [IN]     原文数据的长度
 *           outd     [OUT]    编码后的数据,传NULL时，通过outd_len返回长度
 *           outd_len [IN/OUT] 输入时(outd不为NULL)表示编码后数据的缓冲区长度
 *                             输出时表示编码后的数据长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_Base64Encode(sof_ctx_t *sctx, const kuchar *ind, uint32 ind_len, kuchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 SOF_Base64Decode
 * 功能说明: 数据Base64解码
 * 参数说明: ind      [IN]     Base64编码的数据
 *           ind_len  [IN]     Base64编码的数据长度
 *           outd     [OUT]    解码后的数据,传NULL时，通过outd_len返回长度
 *           outd_len [IN/OUT] 输入时(outd不为NULL)表示解码后数据的缓冲区长度
 *                             输出时表示解码后的数据长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_Base64Decode(sof_ctx_t *sctx, const kuchar *ind, uint32 ind_len, kuchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 Base64EncodeFile
 * 功能说明: 对文件做Base64编码
 * 参数说明: inf            [IN]     文件全路径,以'\0'结尾
 *           coded_data     [OUT]    输出当前文件的Base64编码,以'\0'结尾,传NULL时通过coded_data_len获取长度
 *           coded_data_len [IN/OUT] 输入时(coded_data不为NULL)表示接收数据的缓冲区长度
 *                                   输出时表示Base64编码数据的长度,不包括'\0'
 * 返回值: 0成功 非0失败 
 * 备注  
 ***************************************************/
int SOF_API Base64EncodeFile(sof_ctx_t *sctx, const kuchar *inf, kuchar *outd, uint32 *outd_len);

/***************************************************
 * 函数名称 Base64Decode2File
 * 功能说明: 对数据解码后保存到文件
 * 参数说明: inf            [IN]    文件全路径,以'\0'结尾
 *           coded_data     [IN]    待解码数据
 *           coded_data_len [IN]	待解码数据长度 
 * 返回值: 0成功 非0失败 
 * 备注  
 ***************************************************/
int SOF_API Base64Decode2File(sof_ctx_t *sctx, const kuchar *ind, uint32 ind_len,const kuchar *inf);

enum { 
	// 获取设备信息的类型
	e_Label              = 1,    // 设备标签
	e_FreeSpace          = 2,    // 剩余空间
	e_SerialNumber       = 3,    // 设备序列号
	e_DeviceType         = 4,    // 设备类型
	e_userPinRetryCount  = 5,    // 用户密码重试次数
	e_adminPinRetryCount = 6,    // 管理员密码重试次数
	e_deviceType2        = 7,    // 返回设备类型 硬件设备返回HARD 软设备返回SOFT
	e_devicepridll       = 8,    // 返回对应的扩展库名称
	e_deviceprovider     = 9,    // 返回CSP名称

	e_TotalSpace             = 10, // 设备总空间
	e_userPinTotalRetryCount = 11, // 用户PIN最多重试次数
	e_oldContainerName       = 12, // 当前最旧的容器名称
	e_driverVersion          = 13, // 驱动全路径和版本号
	e_SymAlgCap              = 14, // 支持的分组密码算法
	e_AsymAlgCap             = 15, // 支持的非对称密码算法
	e_deviceType3            = 16, // 设备类型 返回SOFT或USBKEY或ICCARD
	e_support_pki            = 17, // 返回是否支持PKI应用(针对于居民健康卡老卡)
	e_rhcard_userinfo        = 18, // 针对于居民健康卡返回用户信息 姓名&&&身份证号&&&电话&&&
	e_HashAlgCap             = 19, // 支持的摘要算法

	e_KeyType      = 0x73, // 设备类型 RSA 返回10 SM2设备返回20 兼容老的BJCASAC 发证页面用到
	e_vid_pid      = 0x74, // 设备的VID 和 PID兼容老的BJCASAC 发证页面用到
	e_providertype = 0x75, // 返回设备类型 返回SKF、BJCAKey、CSP、CSP11、Soft
	// 设备类型

	e_device_type_soft     = 1,  // 软证书

	e_device_type_usbkey   = 2,  // USBKey设备

	e_device_type_iccard   = 3,  // IC卡设备

	e_device_type_mhcard   = 4,  // 居民健康卡(也是IC卡 不过有的卡不支持PKI应用)

	e_device_type_otpkey   = 5,  // OTP+KEY设备

	e_device_type_cloudkey = 6

};



enum {
        // 用户类型
        e_admin = 0,
        e_user = 1,

        // 权限类型
        e_puser  = 0x10,
        e_padmin = 0x01,
        e_pnone  = 0x00,
        e_pany   = 0x0ff,

        // PIN码校验类型
        e_pintype_password = 1,
        e_pintype_finger = 2,
        e_pintype_otp = 3
};

enum {

	//密钥类型

	e_key_sign = 1, // 签名

	e_key_enc  = 0, // 加密

};

enum {
	//获取证书的信息类型
	e_cert_not_after = 12,
};

// 对称算法信息结构定义
typedef struct SymAlgInfo {
	uint32 ulAlgId_;
	int    nid_;
	uint32 ulKeyLen_;
	uint32 ulIVLen_;
}SymAlgInfo;

/***************************************************
 * 函数名称 GetDeviceCount
 * 功能说明: 获取设备数量
 * 参数说明: dev_count [OUT] 设备数量
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API GetDeviceCount(sof_ctx_t *sctx,uint32 *len);

/***************************************************
 * 函数名称 GetDeviceCountEx
 * 功能说明: 获取设备数量
 * 参数说明: type      [IN]  获取的类型 1 硬设备 2软设备 3全部设备
 *           dev_count [OUT] 设备数量
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API GetDeviceCountEx(sof_ctx_t *sctx, uint32 type, uint32 *dev_count);

/***************************************************
 * 函数名称 GetAllDeviceSN
 * 功能说明: 获取所有设备序列号
 * 参数说明: dev_sn     [OUT]    输出的所有设备序列号,以'\0'结尾,传NULL时通过dev_sn_len获取长度 
 *                               形如 devsn1;devsn2;...
 *           dev_sn_len [IN/OUT] 输入时(dev_sn不为NULL)表示接收所有设备序列号的缓冲区长度
 *                               输出时表示所有设备序列号的长度,不包括'\0'
 * 返回值: 0成功 非0失败 
 * 
 ***************************************************/
int SOF_API  GetAllDeviceSN(sof_ctx_t *sctx,uchar *AllDeviceSN,uint32 *sn_count);

/***************************************************
 * 函数名称 GetAllDeviceSNEx
 * 功能说明: 获取所有设备序列号
 * 参数说明: type       [IN]     获取的类型 1 硬设备 2软设备 3全部设备
 *           dev_sn     [OUT]    输出的所有设备序列号,以'\0'结尾,传NULL时通过dev_sn_len获取长度 
 *                               形如 devsn1;devsn2;...
 *           dev_sn_len [IN/OUT] 输入时(dev_sn不为NULL)表示接收所有设备序列号的缓冲区长度
 *                               输出时表示所有设备序列号的长度,不包括'\0'
 * 返回值: 0成功 非0失败 
 * 备注 
 ***************************************************/
int SOF_API GetAllDeviceSNEx(sof_ctx_t *sctx, uint32 type, uchar *dev_sn, uint32 *dev_sn_len);

/***************************************************
 * 函数名称 GetDeviceSNByIndex
 * 功能说明: 根据索引获取设备序列号
 * 参数说明: index     [IN]      索引号 从0开始
 *           dev_sn     [OUT]    输出的设备序列号,以'\0'结尾,传NULL时通过dev_sn_len获取长度
 *           dev_sn_len [IN/OUT] 输入时(dev_sn不为NULL)表示接收设备序列号的缓冲区长度
 *                               输出时表示设备序列号的长度,不包括'\0'
 * 返回值: 0成功 非0失败 
 * 备注 
 ***************************************************/
int SOF_API GetDeviceSNByIndex(sof_ctx_t *sctx, uint32 index, char *dev_sn, uint32 *dev_sn_len);

/***************************************************
 * 函数名称 IsDeviceExist
 * 功能说明: 判断设备是否存在
 * 参数说明: dev_sn   [IN]  设备序列号,以'\0'结尾
 *           pres     [OUT] 容器存在返回1 不存在返回0 
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API IsDeviceExist(sof_ctx_t *sctx, kchar *dev_sn, uint32 *pres);

/***************************************************
 * 函数名称 GetDeviceInfo
 * 功能说明: 获取设备的详细信息
 * 参数说明: dev_sn   IN]      设备序列号,以'\0'结尾
 *           type     [IN]     信息类型
 *           info     [OUT]    输出的信息,以'\0'结尾,传NULL时通过outd_len获取长度
 *           info_len [IN/OUT] 输入时(info不为NULL)表示接收信息的缓冲区长度
 *                             输出时表示信息的长度,不包括'\0'
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API GetDeviceInfo(sof_ctx_t *sctx, kchar *dev_sn, uint32 type, char *info, uint32 *info_len);

/***************************************************
 * 函数名称 ChangeAdminPass
 * 功能说明: 修改管理员密码
 * 参数说明: dev_sn   [IN] 设备序列号,以'\0'结尾
 *           old_pass [IN] 旧管理员密码,以'\0'结尾
 *           new_pass [IN] 新管理员密码,以'\0'结尾
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API ChangeAdminPass(sof_ctx_t *sctx,uchar * sDeviceSN, uchar * sOldPass, uchar * sNewPass);

/***************************************************
 * 函数名称 UnlockUserPass
 * 功能说明: 解锁用户口令
 * 参数说明: dev_sn        [IN] 设备序列号,以'\0'结尾
 *           admin_pass    [IN] 管理员密码,以'\0'结尾
 *           new_user_pass [IN] 新用户密码,以'\0'结尾
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API UnlockUserPass(sof_ctx_t *sctx, kchar *dev_sn, kchar *admin_pass, kchar *new_user_pass);

/***************************************************
 * 函数名称 UnlockUserPassEx
 * 功能说明: 解锁用户口令
 * 参数说明: dev_sn        [IN] 设备序列号,以'\0'结尾
 *           admin_pass    [IN] 管理员密码,以'\0'结尾
 *           new_user_pass [IN] 新用户密码,以'\0'结尾
 * 返回值: 0成功 非0失败 
 * 说明：sNewUserPass 是编码后的新用户口令，编码规则需先 base64,然后倒序,再

 *			base64。编码前长度为 6~16 位， 取值范围： 0x20 至 0x7e
 ***************************************************/
int SOF_API UnlockUserPassEx(sof_ctx_t *sctx,uchar * sDeviceSN, uchar * sAdminPass, uchar * sNewUserPass);

/***************************************************
 * 函数名称 GenerateKeyPair
 * 功能说明: 产生密钥对
 * 参数说明: dev_sn   [IN] 设备序列号,以'\0'结尾
 *           con_name [IN] 容器名,以'\0'结尾
 *           keytype  [IN] 密钥类型 1:RSA-1024 2:RSA-2048 3:SM2-256
 *           bsign    [IN] 密钥位置 1:签名 0:加密
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API GenerateKeyPair(sof_ctx_t *sctx,uchar * sDeviceSN,uchar * sContainerName);

/***************************************************
 * 函数名称 ExportPubKey
 * 功能说明: 导出公钥
 * 参数说明: dev_sn         [IN]     设备序列号,以'\0'结尾
 *           con_name       [IN]     容器名,以'\0'结尾
 *           bsign          [IN]     密钥位置 1:签名 0:加密
 *           der_pubkey     [OUT]    输出公钥的Der编码,传NULL时通过der_pubkey_len获取长度
 *           der_pubkey_len [IN/OUT] 输入时(der_pubkey不为NULL)表示接收公钥的缓冲区长度
 *                                   输出时表示公钥的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API ExportPubKey(sof_ctx_t *sctx, kchar *dev_sn, kchar *con_name, 
				 uint32 bsign, uchar *der_pubkey, uint32 *der_pubkey_len);

/***************************************************
 * 函数名称 ExportPKCS10
 * 功能说明: 导出PKCS10证书请求
 * 参数说明: dev_sn      [IN]     设备序列号,以'\0'结尾
 *           con_name    [IN]     容器名,以'\0'结尾
 *           bsign       [IN]     密钥位置 1:签名 0:加密
 *           dn          [IN]     DN,以'\0'结尾
 *                                形如"CN=name, OU=BJCA R&D, O=BJCA, L=\\,北京, ST=bj, C=CN, E=mail@bjca.com,"
 *                                如果DN项中有逗号(,)，前边加上\进行转义
 *           der_p10     [OUT]    输出Der编码的P10,传NULL时通过der_p10_len获取长度
 *           der_p10_len [IN/OUT] 输入时(der_pubkey不为NULL)表示接收P10的缓冲区长度
 *                                输出时表示P10的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API ExportPKCS10(sof_ctx_t *sctx, kchar *dev_sn, kchar *con_name, 
						 uint32 bsign, kchar *dn, uchar *der_p10, uint32 *der_p10_len);

/***************************************************
 * 函数名称 ImportSignCert
 * 功能说明: 导入签名证书
 * 参数说明: dev_sn        [IN] 设备序列号,以'\0'结尾
 *           con_name      [IN] 容器名,以'\0'结尾
 *           user_cert     [IN] Der编码的签名证书
 *           user_cert_len [IN] Der编码的签名证书长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API ImportSignCert(sof_ctx_t *sctx, kchar *dev_sn, kchar *con_name, uchar *user_cert, uint32 user_cert_len);

/***************************************************
 * 函数名称 ImportEncCert
 * 功能说明: 导入加密证书和加密密钥对
 * 参数说明: dev_sn            [IN] 设备序列号,以'\0'结尾
 *           con_name          [IN] 容器名,以'\0'结尾
 *           user_cert         [IN] Der编码的签名证书
 *           user_cert_len     [IN] Der编码的签名证书长度
 *           cipher_prikey     [IN] 加密密钥对的密文
 *           cipher_prikey_len [IN] 加密密钥对的密文长度
 *           sym_algo          [IN] 对称算法
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API ImportEncCert(sof_ctx_t *sctx, kchar *dev_sn, kchar *con_name, 
				  kuchar *user_cert, uint32 user_cert_len, 
				  kuchar *cipher_prikey, uint32 cipher_prikey_len, uint32 sym_algo);

/***************************************************
 * 函数名称 EnumFilesInDevice
 * 功能说明: 枚举设备内的所有文件
 * 参数说明: dev_sn     [IN]  设备序列号,以'\0'结尾
 *           files      [OUT]    输出所有文件名称 中间以&&&分割 例如返回 file1&&&file2&&&....filen&&&
 *                               本参数输入NULL是 通过files_len返回长度
 *           files_len  [IN/OUT] 输入时(files不为NULL)表示接收所有文件名称的缓冲区长度
 *                               输出时表示所有文件名称的长度,不包括'\0'
 * 返回值: 0成功 非0失败 
 * 
 ***************************************************/
int SOF_API EnumFilesInDevice(sof_ctx_t *sctx, kchar *dev_sn,
				  char *files, uint32 *files_len);

/***************************************************
 * 函数名称 SOF_ReadFile
 * 功能说明: 在设备中读取文件
 * 参数说明: dev_sn           [IN]     设备序列号,以'\0'结尾
 *           file_name        [IN]     文件名,以'\0'结尾
 *           file_content     [OUT]    文件内容,传NULL时通过file_content_len获取长度 
 *           file_content_len [IN/OUT] 输入时(file_content不为NULL)表示接收文件内容的缓冲区长度
 *                                     输出时表示文件内容的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_ReadFile(sof_ctx_t *sctx,uchar * sDeviceSN, uchar  *file_name, 
			 uchar *ind, uint32 *ind_len);

/***************************************************
 * 函数名称 SOF_WriteFile
 * 功能说明: 向文件内容写入到设备中
 * 参数说明: dev_sn           [IN] 设备序列号,以'\0'结尾
 *           file_name        [IN] 文件名,以'\0'结尾
 *           file_content     [IN] 文件内容
 *           file_content_len [IN] 文件内容的长度
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_WriteFile(sof_ctx_t *sctx, kchar *dev_sn, kchar *file_name, 
			  kuchar *file_content, uint32 file_content_len);

/***************************************************
 * 函数名称 SOF_WriteFileEx
 * 功能说明: 向文件内容写入到设备中
 * 参数说明: dev_sn           [IN] 设备序列号,以'\0'结尾
 *           file_name         [IN] 文件名,以'\0'结尾
 *           file_content      [IN] 文件内容
 *           file_content_len  [IN] 文件内容的长度
 *           bPrivate          [IN] 是否写到私有区
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SOF_WriteFileEx(sof_ctx_t *sctx, kchar *dev_sn, kchar *file_name, 
				kuchar *file_content, uint32 file_content_len, int bPrivate);

/***************************************************
 * 函数名称 GetContainerCount
 * 功能说明: 获取设备中的容器数量
 * 参数说明: dev_sn    [IN]  设备序列号,以'\0'结尾
 *           con_count [OUT] 容器数量 
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API GetContainerCount(sof_ctx_t *sctx, kchar *dev_sn, uint32 *con_count);

/***************************************************
 * 函数名称 SOF_GetAllContainerName
 * 功能说明: 获取设备中所有容器名字
 * 参数说明: dev_sn            [IN]    设备序列号,以'\0'结尾
 *           all_con_name      [OUT]   输出的所有容器名称,以'\0'结尾,传NULL时通过all_con_name_len获取长度 
 *                                     形如 container1&&&container2&&&...
 *           all_con_name_len [IN/OUT] 输入时(outd不为NULL)表示接收所有容器名的缓冲区长度
 *                                     输出时表示所有容器名的长度,不包括'\0'
 * 返回值: 0成功 非0失败 
 * 备注 
 ***************************************************/
int SOF_API SOF_GetAllContainerName(sof_ctx_t *sctx, kchar *dev_sn, char *all_con_name, uint32 *all_con_name_len);

/***************************************************
 * 函数名称 IsContainerExist
 * 功能说明: 判断容器是否存在
 * 参数说明: dev_sn   [IN]  设备序列号,以'\0'结尾
 *           con_name [IN]  容器名,以'\0'结尾
 *           pres     [OUT] 容器存在返回1 不存在返回0 
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API IsContainerExist(sof_ctx_t *sctx, kchar *dev_sn, kchar *con_name, uint32 *pres);

/***************************************************
 * 函数名称 DeleteContainer
 * 功能说明: 删除容器
 * 参数说明: dev_sn   [IN]  设备序列号,以'\0'结尾
 *           con_name [IN]  容器名,以'\0'结尾
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API DeleteContainer(sof_ctx_t *sctx, kchar *dev_sn, kchar *con_name);

/***************************************************
 * 函数名称 DeleteOldContainer
 * 功能说明: 删除设备中最旧的容器
 * 参数说明: dev_sn    [IN]  设备序列号,以'\0'结尾
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API DeleteOldContainer(sof_ctx_t *sctx, kchar *dev_sn);

/***************************************************
 * 函数名称 InitDevice
 * 功能说明: 格式化设备
 * 参数说明: dev_sn     [IN]  设备序列号,以'\0'结尾
 *           admin_pass [IN]  管理员口令,以'\0'结尾
 *           user_pass  [IN]  用户口令,以'\0'结尾
 *           key_label  [IN]  标签,以'\0'结尾
 *           admin_maxretry  [IN]  管理员口令最多重试次数
 *           user_maxretry   [IN]  用户口令最多重试次数
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API InitDevice(sof_ctx_t *sctx, kchar *dev_sn, kchar *admin_pass);

int SOF_API InitDeviceEx(sof_ctx_t *sctx, kchar *dev_sn, kchar *admin_pass, 
			 kchar *user_pass, kchar *key_label, uint32 admin_maxretry, uint32 user_maxretry);

/***************************************************
 * 函数名称 GetENVSN
 * 功能说明: 获取当前在用的容器名
 * 参数说明: dev_sn    [IN]     设备序列号,以'\0'结尾
 *           envsn     [OUT]    输出当前在用的容器名,以'\0'结尾,传NULL时通过envsn_len获取长度
 *           envsn_len [IN/OUT] 输入时(envsn不为NULL)表示接收容器名的缓冲区长度
 *                              输出时表示容器名的长度,不包括'\0'
 * 返回值: 0成功 非0失败 
 * 备注 
 ***************************************************/
int SOF_API GetENVSN(sof_ctx_t *sctx, kchar *dev_sn, char *envsn, uint32 *envsn_len);

/***************************************************
 * 函数名称 SetENVSN
 * 功能说明: 设置当前在用的容器名
 * 参数说明: dev_sn  [IN]  设备序列号,以'\0'结尾
 *           envsn   [IN]  容器名,以'\0'结尾
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SetENVSN(sof_ctx_t *sctx, kchar *dev_sn, kchar *envsn);

/***************************************************
 * 函数名称 SetUserConfig
 * 功能说明: 设置用户配置信息
 * 参数说明: type   [IN] 配置类型       4 表示添加用户可信的 base64编码CA 证书
 *		         		5 表示添加用户可信的 base64编码CRL
 *	               			6 表示添加 base64编码P7B 证书链
 *           config [IN] 设置的参数,以'\0'结尾 
 *                               
 * 返回值: 0成功 非0失败 
 ***************************************************/
int SOF_API SetUserConfig(sof_ctx_t *sctx, uint32 type, kchar *config);

#ifdef __cplusplus
}
#endif

#endif // ifndef _XTXAPP_C_UTIL_H_
